(ns wanijo.schema.middleware
  (:require [ring.util.response :as resp]
            [wanijo.infra.routing :refer [path]]
            [wanijo.schema.db :as db]))

(defn wrap-user-schemas! [handler]
  (fn [req]
    (if-let [uuid (get-in req [:session :uuid])]
      (handler (assoc-in req
                         [:session :schemas]
                         (db/all-readable-by! uuid)))
      (handler req))))

(defn write-permission-middleware! [schema-fn]
  (fn [handler]
    (fn [req]
      (let [uuid (schema-fn req)]
        (if (db/has-user-write-permissions? uuid (-> req :session :uuid))
          (handler req)
          (assoc
           (resp/redirect (path :schema-show {:uuid uuid}))
           :flash ["No write permission for schema"]))))))

(defn wrap-allowed-to-write!
  ([schema-fn]
   (write-permission-middleware! schema-fn))
  ([]
   (write-permission-middleware! #(get-in % [:params :uuid]))))

(defn wrap-allowed-to-read!
  [schema-fn]
  (fn [handler]
    (fn [req]
      (let [uuid (schema-fn req)]
        (if (db/has-user-read-permissions? uuid (-> req :session :uuid))
          (handler req)
          (assoc (resp/redirect (path :home))
            :flash ["No read permission for schema"]))))))
